-- something that represents a level camera, polygon sotrage and renderer

local Level={
	-- camera position, rotation and zoom
	x=0,
	y=0,
	angle=0,
	zoom=1,
	
	-- for best result i apply only transform which is a lot of calculation that can be made during and update instead every time
	-- per draw frame!
	transform=love.math.newTransform(),
	
	-- polygons to draw
	polygons={},
	
	help=[[
Demo made by Rigachupe (version 2).
Both polygons are a 4x4 tile made from 2 triangles with a pivot point [0,0] in the middle. Each uses it's own texture.
a,s,d,w - move camera
z,x - zoom camera
f - toggle filter nearest/linear
i,j,k,l - move polygon
o,p - rotate polygon
c - toggle wireframe on/off
escape - exit
	]]
}

function Level.test()
	-- make some rock quad with 15 degree rotation
	local polygon1=Polygon.test("data/rock.png",true)
	Polygon.setRotation(polygon1,math.rad(15))
	Polygon.updateTransform(polygon1)
	
	-- make some sand quad with position away from rock quad
	local polygon2=Polygon.test("data/sand.png",false)
	Polygon.setPosition(polygon2,200,-5)
	Polygon.updateTransform(polygon2)
	
	-- add polygons to level
	Level.addPolygon(polygon1)
	Level.addPolygon(polygon2)
	
	-- set best camera view for level
	local sw,sh=love.graphics.getDimensions()
	Level.setCamera(sw/2,sh/2,1)
end

-- add the polygon to level polygons
function Level.addPolygon(polygon)
	table.insert(Level.polygons,polygon)
end

-- set the camera to position x,y and the z is like a zoom factor
function Level.setCamera(x,y,zoom)
	Level.x=x
	Level.y=y
	Level.zoom=zoom
	Level.updateCamera()
end

-- update camera only when the camera position changes
function Level.updateCamera()
	-- note : i should like calculate the box of the view and then draw and update only polygons that are in the view
	Level.transform:reset()
	Level.transform:translate(Level.x,Level.y)
	Level.transform:rotate(Level.angle)
	Level.transform:scale(Level.zoom)
end

-- draw the level with polygons
function Level.draw()
	-- position and zoom the camera
	love.graphics.applyTransform(Level.transform)
	
	-- set the wireframe
	if Window.wireframe then
		love.graphics.setWireframe(true)
	end
	
	-- render polygons
	love.graphics.setColor(1,1,1,1)
	for index,polygon in ipairs(Level.polygons) do
		Polygon.draw(Level.shader,polygon)
	end
	
	-- turn of the wireframe
	love.graphics.setWireframe(false)
	
	-- restore transform to origin
	love.graphics.origin()
	
	-- and i can render normally some UI components
	love.graphics.setColor(1,1,1,1)
	love.graphics.print("Camera at:"..Level.x..","..Level.y..","..Level.zoom,10,10)
	love.graphics.print(Level.help,10,30)
end

-- the keys that handle the demo output
function Level.keypressed(key)
	if key=="a" then
		Level.x=Level.x-1
		Level.updateCamera()
	elseif key=="d" then
		Level.x=Level.x+1
		Level.updateCamera()
	elseif key=="w" then
		Level.y=Level.y-1
		Level.updateCamera()
	elseif key=="s" then
		Level.y=Level.y+1
		Level.updateCamera()
	elseif key=="z" then
		Level.zoom=Level.zoom+0.1
		if Level.zoom>10 then
			Level.zoom=10
		end
		Level.updateCamera()
	elseif key=="x" then
		Level.zoom=Level.zoom-0.1
		if Level.zoom<1 then
			Level.zoom=1
		end
		Level.updateCamera()
	elseif key=="o" then
		local p=Level.polygons[1]
		p.angle=p.angle-0.05
		Polygon.updateTransform(p)
	elseif key=="p" then
		local p=Level.polygons[1]
		p.angle=p.angle+0.05
		Polygon.updateTransform(p)
	elseif key=="j" then
		local p=Level.polygons[1]
		p.x=p.x-0.5
		Polygon.updateTransform(p)
	elseif key=="l" then
		local p=Level.polygons[1]
		p.x=p.x+0.5
		Polygon.updateTransform(p)
	elseif key=="i" then
		local p=Level.polygons[1]
		p.y=p.y-0.5
		Polygon.updateTransform(p)
	elseif key=="k" then
		local p=Level.polygons[1]
		p.y=p.y+0.5
		Polygon.updateTransform(p)
	elseif key=="f" then
		Window.toggleDefaultFilter()
	elseif key=="c" then
		Window.toggleWireframe()
	elseif key=="escape" then
		love.event.quit(0)
	end
end

return Level